@@ -16,26 +16,27 @@
 
 package com.google.common.collect;
 
+import static com.google.common.collect.Platform.checkGwtRpcEnabled;
+
 import com.google.gwt.user.client.rpc.SerializationStreamReader;
 import com.google.gwt.user.client.rpc.SerializationStreamWriter;
 
 /**
  * This class implements the GWT serialization of {@link UsingToStringOrdering}.
- * 
+ *
  * @author Chris Povirk
  */
 public class UsingToStringOrdering_CustomFieldSerializer {
 
-  public static void deserialize(SerializationStreamReader reader,
-      UsingToStringOrdering instance) {
-  }
+  public static void deserialize(
+      SerializationStreamReader reader, UsingToStringOrdering instance) {}
 
-  public static UsingToStringOrdering instantiate(
-      SerializationStreamReader reader) {
+  public static UsingToStringOrdering instantiate(SerializationStreamReader reader) {
+    checkGwtRpcEnabled();
     return UsingToStringOrdering.INSTANCE;
   }
 
-  public static void serialize(SerializationStreamWriter writer,
-      UsingToStringOrdering instance) {
+  public static void serialize(SerializationStreamWriter writer, UsingToStringOrdering instance) {
+    checkGwtRpcEnabled();
   }
 }
