@@ -19,12 +19,12 @@ package com.google.common.testing;
 import static com.google.common.base.Preconditions.checkArgument;
 
 import com.google.common.annotations.Beta;
+import com.google.common.annotations.GwtIncompatible;
 import com.google.common.base.CharMatcher;
 import com.google.common.base.Charsets;
 import com.google.common.base.Defaults;
 import com.google.common.base.Equivalence;
 import com.google.common.base.Joiner;
-import com.google.common.base.Optional;
 import com.google.common.base.Predicate;
 import com.google.common.base.Predicates;
 import com.google.common.base.Splitter;
@@ -48,8 +48,6 @@ import com.google.common.collect.ImmutableSortedSet;
 import com.google.common.collect.ImmutableTable;
 import com.google.common.collect.Iterators;
 import com.google.common.collect.ListMultimap;
-import com.google.common.collect.MapConstraint;
-import com.google.common.collect.MapConstraints;
 import com.google.common.collect.MapDifference;
 import com.google.common.collect.Maps;
 import com.google.common.collect.Multimap;
@@ -76,7 +74,6 @@ import com.google.common.io.CharSource;
 import com.google.common.primitives.Primitives;
 import com.google.common.primitives.UnsignedInteger;
 import com.google.common.primitives.UnsignedLong;
-
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.File;
@@ -121,11 +118,16 @@ import java.util.Locale;
 import java.util.Map;
 import java.util.NavigableMap;
 import java.util.NavigableSet;
+import java.util.Optional;
+import java.util.OptionalDouble;
+import java.util.OptionalInt;
+import java.util.OptionalLong;
 import java.util.Queue;
 import java.util.Random;
 import java.util.Set;
 import java.util.SortedMap;
 import java.util.SortedSet;
+import java.util.UUID;
 import java.util.concurrent.BlockingDeque;
 import java.util.concurrent.BlockingQueue;
 import java.util.concurrent.ConcurrentHashMap;
@@ -144,13 +146,13 @@ import java.util.logging.Logger;
 import java.util.regex.MatchResult;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
-
-import javax.annotation.Nullable;
+import java.util.stream.Stream;
+import org.checkerframework.checker.nullness.qual.Nullable;
 
 /**
  * Supplies an arbitrary "default" instance for a wide range of types, often useful in testing
  * utilities.
- * 
+ *
  * <p>Covers arrays, enums and common types defined in {@code java.lang}, {@code java.lang.reflect},
  * {@code java.io}, {@code java.nio}, {@code java.math}, {@code java.util}, {@code
  * java.util.concurrent}, {@code java.util.regex}, {@code com.google.common.base}, {@code
@@ -158,25 +160,28 @@ import javax.annotation.Nullable;
  * exposes at least one public static final constant of the same type, one of the constants will be
  * used; or if the class exposes a public parameter-less constructor then it will be "new"d and
  * returned.
- * 
+ *
  * <p>All default instances returned by {@link #get} are generics-safe. Clients won't get type
  * errors for using {@code get(Comparator.class)} as a {@code Comparator<Foo>}, for example.
- * Immutable empty instances are returned for collection types; {@code ""} for string;
- * {@code 0} for number types; reasonable default instance for other stateless types. For mutable
- * types, a fresh instance is created each time {@code get()} is called.
+ * Immutable empty instances are returned for collection types; {@code ""} for string; {@code 0} for
+ * number types; reasonable default instance for other stateless types. For mutable types, a fresh
+ * instance is created each time {@code get()} is called.
  *
  * @author Kevin Bourrillion
  * @author Ben Yu
  * @since 12.0
  */
 @Beta
+@GwtIncompatible
 public final class ArbitraryInstances {
 
-  private static final Ordering<Field> BY_FIELD_NAME = new Ordering<Field>() {
-    @Override public int compare(Field left, Field right) {
-      return left.getName().compareTo(right.getName());
-    }
-  };
+  private static final Ordering<Field> BY_FIELD_NAME =
+      new Ordering<Field>() {
+        @Override
+        public int compare(Field left, Field right) {
+          return left.getName().compareTo(right.getName());
+        }
+      };
 
   /**
    * Returns a new {@code MatchResult} that corresponds to a successful match. Apache Harmony (used
@@ -189,113 +194,123 @@ public final class ArbitraryInstances {
     return matcher.toMatchResult();
   }
 
-  private static final ClassToInstanceMap<Object> DEFAULTS = ImmutableClassToInstanceMap.builder()
-      // primitives
-      .put(Object.class, "")
-      .put(Number.class, 0)
-      .put(UnsignedInteger.class, UnsignedInteger.ZERO)
-      .put(UnsignedLong.class, UnsignedLong.ZERO)
-      .put(BigInteger.class, BigInteger.ZERO)
-      .put(BigDecimal.class, BigDecimal.ZERO)
-      .put(CharSequence.class, "")
-      .put(String.class, "")
-      .put(Pattern.class, Pattern.compile(""))
-      .put(MatchResult.class, newMatchResult())
-      .put(TimeUnit.class, TimeUnit.SECONDS)
-      .put(Charset.class, Charsets.UTF_8)
-      .put(Currency.class, Currency.getInstance(Locale.US))
-      .put(Locale.class, Locale.US)
-      // common.base
-      .put(CharMatcher.class, CharMatcher.NONE)
-      .put(Joiner.class, Joiner.on(','))
-      .put(Splitter.class, Splitter.on(','))
-      .put(Optional.class, Optional.absent())
-      .put(Predicate.class, Predicates.alwaysTrue())
-      .put(Equivalence.class, Equivalence.equals())
-      .put(Ticker.class, Ticker.systemTicker())
-      .put(Stopwatch.class, Stopwatch.createUnstarted())
-      // io types
-      .put(InputStream.class, new ByteArrayInputStream(new byte[0]))
-      .put(ByteArrayInputStream.class, new ByteArrayInputStream(new byte[0]))
-      .put(Readable.class, new StringReader(""))
-      .put(Reader.class, new StringReader(""))
-      .put(StringReader.class, new StringReader(""))
-      .put(Buffer.class, ByteBuffer.allocate(0))
-      .put(CharBuffer.class, CharBuffer.allocate(0))
-      .put(ByteBuffer.class, ByteBuffer.allocate(0))
-      .put(ShortBuffer.class, ShortBuffer.allocate(0))
-      .put(IntBuffer.class, IntBuffer.allocate(0))
-      .put(LongBuffer.class, LongBuffer.allocate(0))
-      .put(FloatBuffer.class, FloatBuffer.allocate(0))
-      .put(DoubleBuffer.class, DoubleBuffer.allocate(0))
-      .put(File.class, new File(""))
-      .put(ByteSource.class, ByteSource.empty())
-      .put(CharSource.class, CharSource.empty())
-      .put(ByteSink.class, NullByteSink.INSTANCE)
-      .put(CharSink.class, NullByteSink.INSTANCE.asCharSink(Charsets.UTF_8))
-      // All collections are immutable empty. So safe for any type parameter.
-      .put(Iterator.class, ImmutableSet.of().iterator())
-      .put(PeekingIterator.class, Iterators.peekingIterator(Iterators.emptyIterator()))
-      .put(ListIterator.class, ImmutableList.of().listIterator())
-      .put(Iterable.class, ImmutableSet.of())
-      .put(Collection.class, ImmutableList.of())
-      .put(ImmutableCollection.class, ImmutableList.of())
-      .put(List.class, ImmutableList.of())
-      .put(ImmutableList.class, ImmutableList.of())
-      .put(Set.class, ImmutableSet.of())
-      .put(ImmutableSet.class, ImmutableSet.of())
-      .put(SortedSet.class, ImmutableSortedSet.of())
-      .put(ImmutableSortedSet.class, ImmutableSortedSet.of())
-      .put(NavigableSet.class, Sets.unmodifiableNavigableSet(Sets.newTreeSet()))
-      .put(Map.class, ImmutableMap.of())
-      .put(ImmutableMap.class, ImmutableMap.of())
-      .put(SortedMap.class, ImmutableSortedMap.of())
-      .put(ImmutableSortedMap.class, ImmutableSortedMap.of())
-      .put(NavigableMap.class, Maps.unmodifiableNavigableMap(Maps.newTreeMap()))
-      .put(Multimap.class, ImmutableMultimap.of())
-      .put(ImmutableMultimap.class, ImmutableMultimap.of())
-      .put(ListMultimap.class, ImmutableListMultimap.of())
-      .put(ImmutableListMultimap.class, ImmutableListMultimap.of())
-      .put(SetMultimap.class, ImmutableSetMultimap.of())
-      .put(ImmutableSetMultimap.class, ImmutableSetMultimap.of())
-      .put(SortedSetMultimap.class, Multimaps.unmodifiableSortedSetMultimap(TreeMultimap.create()))
-      .put(Multiset.class, ImmutableMultiset.of())
-      .put(ImmutableMultiset.class, ImmutableMultiset.of())
-      .put(SortedMultiset.class, ImmutableSortedMultiset.of())
-      .put(ImmutableSortedMultiset.class, ImmutableSortedMultiset.of())
-      .put(BiMap.class, ImmutableBiMap.of())
-      .put(ImmutableBiMap.class, ImmutableBiMap.of())
-      .put(Table.class, ImmutableTable.of())
-      .put(ImmutableTable.class, ImmutableTable.of())
-      .put(RowSortedTable.class, Tables.unmodifiableRowSortedTable(TreeBasedTable.create()))
-      .put(ClassToInstanceMap.class, ImmutableClassToInstanceMap.builder().build())
-      .put(ImmutableClassToInstanceMap.class, ImmutableClassToInstanceMap.builder().build())
-      .put(Comparable.class, ByToString.INSTANCE)
-      .put(Comparator.class, AlwaysEqual.INSTANCE)
-      .put(Ordering.class, AlwaysEqual.INSTANCE)
-      .put(Range.class, Range.all())
-      .put(MapConstraint.class, MapConstraints.notNull())
-      .put(MapDifference.class, Maps.difference(ImmutableMap.of(), ImmutableMap.of()))
-      .put(SortedMapDifference.class,
-          Maps.difference(ImmutableSortedMap.of(), ImmutableSortedMap.of()))
-      // reflect
-      .put(AnnotatedElement.class, Object.class)
-      .put(GenericDeclaration.class, Object.class)
-      .put(Type.class, Object.class)
-      .build();
+  private static final ClassToInstanceMap<Object> DEFAULTS =
+      ImmutableClassToInstanceMap.builder()
+          // primitives
+          .put(Object.class, "")
+          .put(Number.class, 0)
+          .put(UnsignedInteger.class, UnsignedInteger.ZERO)
+          .put(UnsignedLong.class, UnsignedLong.ZERO)
+          .put(BigInteger.class, BigInteger.ZERO)
+          .put(BigDecimal.class, BigDecimal.ZERO)
+          .put(CharSequence.class, "")
+          .put(String.class, "")
+          .put(Pattern.class, Pattern.compile(""))
+          .put(MatchResult.class, newMatchResult())
+          .put(TimeUnit.class, TimeUnit.SECONDS)
+          .put(Charset.class, Charsets.UTF_8)
+          .put(Currency.class, Currency.getInstance(Locale.US))
+          .put(Locale.class, Locale.US)
+          .put(Optional.class, Optional.empty())
+          .put(OptionalInt.class, OptionalInt.empty())
+          .put(OptionalLong.class, OptionalLong.empty())
+          .put(OptionalDouble.class, OptionalDouble.empty())
+          .put(UUID.class, UUID.randomUUID())
+          // common.base
+          .put(CharMatcher.class, CharMatcher.none())
+          .put(Joiner.class, Joiner.on(','))
+          .put(Splitter.class, Splitter.on(','))
+          .put(com.google.common.base.Optional.class, com.google.common.base.Optional.absent())
+          .put(Predicate.class, Predicates.alwaysTrue())
+          .put(Equivalence.class, Equivalence.equals())
+          .put(Ticker.class, Ticker.systemTicker())
+          .put(Stopwatch.class, Stopwatch.createUnstarted())
+          // io types
+          .put(InputStream.class, new ByteArrayInputStream(new byte[0]))
+          .put(ByteArrayInputStream.class, new ByteArrayInputStream(new byte[0]))
+          .put(Readable.class, new StringReader(""))
+          .put(Reader.class, new StringReader(""))
+          .put(StringReader.class, new StringReader(""))
+          .put(Buffer.class, ByteBuffer.allocate(0))
+          .put(CharBuffer.class, CharBuffer.allocate(0))
+          .put(ByteBuffer.class, ByteBuffer.allocate(0))
+          .put(ShortBuffer.class, ShortBuffer.allocate(0))
+          .put(IntBuffer.class, IntBuffer.allocate(0))
+          .put(LongBuffer.class, LongBuffer.allocate(0))
+          .put(FloatBuffer.class, FloatBuffer.allocate(0))
+          .put(DoubleBuffer.class, DoubleBuffer.allocate(0))
+          .put(File.class, new File(""))
+          .put(ByteSource.class, ByteSource.empty())
+          .put(CharSource.class, CharSource.empty())
+          .put(ByteSink.class, NullByteSink.INSTANCE)
+          .put(CharSink.class, NullByteSink.INSTANCE.asCharSink(Charsets.UTF_8))
+          // All collections are immutable empty. So safe for any type parameter.
+          .put(Iterator.class, ImmutableSet.of().iterator())
+          .put(PeekingIterator.class, Iterators.peekingIterator(ImmutableSet.of().iterator()))
+          .put(ListIterator.class, ImmutableList.of().listIterator())
+          .put(Iterable.class, ImmutableSet.of())
+          .put(Collection.class, ImmutableList.of())
+          .put(ImmutableCollection.class, ImmutableList.of())
+          .put(List.class, ImmutableList.of())
+          .put(ImmutableList.class, ImmutableList.of())
+          .put(Set.class, ImmutableSet.of())
+          .put(ImmutableSet.class, ImmutableSet.of())
+          .put(SortedSet.class, ImmutableSortedSet.of())
+          .put(ImmutableSortedSet.class, ImmutableSortedSet.of())
+          .put(NavigableSet.class, Sets.unmodifiableNavigableSet(Sets.newTreeSet()))
+          .put(Map.class, ImmutableMap.of())
+          .put(ImmutableMap.class, ImmutableMap.of())
+          .put(SortedMap.class, ImmutableSortedMap.of())
+          .put(ImmutableSortedMap.class, ImmutableSortedMap.of())
+          .put(NavigableMap.class, Maps.unmodifiableNavigableMap(Maps.newTreeMap()))
+          .put(Multimap.class, ImmutableMultimap.of())
+          .put(ImmutableMultimap.class, ImmutableMultimap.of())
+          .put(ListMultimap.class, ImmutableListMultimap.of())
+          .put(ImmutableListMultimap.class, ImmutableListMultimap.of())
+          .put(SetMultimap.class, ImmutableSetMultimap.of())
+          .put(ImmutableSetMultimap.class, ImmutableSetMultimap.of())
+          .put(
+              SortedSetMultimap.class,
+              Multimaps.unmodifiableSortedSetMultimap(TreeMultimap.create()))
+          .put(Multiset.class, ImmutableMultiset.of())
+          .put(ImmutableMultiset.class, ImmutableMultiset.of())
+          .put(SortedMultiset.class, ImmutableSortedMultiset.of())
+          .put(ImmutableSortedMultiset.class, ImmutableSortedMultiset.of())
+          .put(BiMap.class, ImmutableBiMap.of())
+          .put(ImmutableBiMap.class, ImmutableBiMap.of())
+          .put(Table.class, ImmutableTable.of())
+          .put(ImmutableTable.class, ImmutableTable.of())
+          .put(RowSortedTable.class, Tables.unmodifiableRowSortedTable(TreeBasedTable.create()))
+          .put(ClassToInstanceMap.class, ImmutableClassToInstanceMap.builder().build())
+          .put(ImmutableClassToInstanceMap.class, ImmutableClassToInstanceMap.builder().build())
+          .put(Comparable.class, ByToString.INSTANCE)
+          .put(Comparator.class, AlwaysEqual.INSTANCE)
+          .put(Ordering.class, AlwaysEqual.INSTANCE)
+          .put(Range.class, Range.all())
+          .put(MapDifference.class, Maps.difference(ImmutableMap.of(), ImmutableMap.of()))
+          .put(
+              SortedMapDifference.class,
+              Maps.difference(ImmutableSortedMap.of(), ImmutableSortedMap.of()))
+          // reflect
+          .put(AnnotatedElement.class, Object.class)
+          .put(GenericDeclaration.class, Object.class)
+          .put(Type.class, Object.class)
+          .build();
 
   /**
-   * type -> implementation. Inherently mutable interfaces and abstract classes are mapped to their
+   * type → implementation. Inherently mutable interfaces and abstract classes are mapped to their
    * default implementations and are "new"d upon get().
    */
   private static final ConcurrentMap<Class<?>, Class<?>> implementations = Maps.newConcurrentMap();
 
   private static <T> void setImplementation(Class<T> type, Class<? extends T> implementation) {
     checkArgument(type != implementation, "Don't register %s to itself!", type);
-    checkArgument(!DEFAULTS.containsKey(type),
-        "A default value was already registered for %s", type);
-    checkArgument(implementations.put(type, implementation) == null,
-        "Implementation for %s was already registered", type);
+    checkArgument(
+        !DEFAULTS.containsKey(type), "A default value was already registered for %s", type);
+    checkArgument(
+        implementations.put(type, implementation) == null,
+        "Implementation for %s was already registered",
+        type);
   }
 
   static {
@@ -311,8 +326,8 @@ public final class ArbitraryInstances {
     setImplementation(PrintWriter.class, Dummies.InMemoryPrintWriter.class);
     setImplementation(Queue.class, ArrayDeque.class);
     setImplementation(Random.class, Dummies.DeterministicRandom.class);
-    setImplementation(ScheduledThreadPoolExecutor.class,
-        Dummies.DummyScheduledThreadPoolExecutor.class);
+    setImplementation(
+        ScheduledThreadPoolExecutor.class, Dummies.DummyScheduledThreadPoolExecutor.class);
     setImplementation(ThreadPoolExecutor.class, Dummies.DummyScheduledThreadPoolExecutor.class);
     setImplementation(Writer.class, StringWriter.class);
     setImplementation(Runnable.class, Dummies.DummyRunnable.class);
@@ -321,18 +336,17 @@ public final class ArbitraryInstances {
   }
 
   @SuppressWarnings("unchecked") // it's a subtype map
-  @Nullable
-  private static <T> Class<? extends T> getImplementation(Class<T> type) {
+  private static <T> @Nullable Class<? extends T> getImplementation(Class<T> type) {
     return (Class<? extends T>) implementations.get(type);
   }
 
   private static final Logger logger = Logger.getLogger(ArbitraryInstances.class.getName());
 
   /**
-   * Returns an arbitrary instance for {@code type}, or {@code null} if no arbitrary instance can
-   * be determined.
+   * Returns an arbitrary instance for {@code type}, or {@code null} if no arbitrary instance can be
+   * determined.
    */
-  @Nullable public static <T> T get(Class<T> type) {
+  public static <T> @Nullable T get(Class<T> type) {
     T defaultValue = DEFAULTS.getInstance(type);
     if (defaultValue != null) {
       return defaultValue;
@@ -341,11 +355,12 @@ public final class ArbitraryInstances {
     if (implementation != null) {
       return get(implementation);
     }
+    if (type == Stream.class) {
+      return type.cast(Stream.empty());
+    }
     if (type.isEnum()) {
       T[] enumConstants = type.getEnumConstants();
-      return (enumConstants.length == 0)
-          ? null
-          : enumConstants[0];
+      return (enumConstants.length == 0) ? null : enumConstants[0];
     }
     if (type.isArray()) {
       return createEmptyArray(type);
@@ -366,6 +381,11 @@ public final class ArbitraryInstances {
     constructor.setAccessible(true); // accessibility check is too slow
     try {
       return constructor.newInstance();
+      /*
+       * Do not merge the 2 catch blocks below. javac would infer a type of
+       * ReflectiveOperationException, which Animal Sniffer would reject. (Old versions of
+       * Android don't *seem* to mind, but there might be edge cases of which we're unaware.)
+       */
     } catch (InstantiationException impossible) {
       throw new AssertionError(impossible);
     } catch (IllegalAccessException impossible) {
@@ -376,15 +396,14 @@ public final class ArbitraryInstances {
     }
   }
 
-  @Nullable private static <T> T arbitraryConstantInstanceOrNull(Class<T> type) {
+  private static <T> @Nullable T arbitraryConstantInstanceOrNull(Class<T> type) {
     Field[] fields = type.getDeclaredFields();
     Arrays.sort(fields, BY_FIELD_NAME);
     for (Field field : fields) {
       if (Modifier.isPublic(field.getModifiers())
           && Modifier.isStatic(field.getModifiers())
           && Modifier.isFinal(field.getModifiers())) {
-        if (field.getGenericType() == field.getType()
-            && type.isAssignableFrom(field.getType())) {
+        if (field.getGenericType() == field.getType() && type.isAssignableFrom(field.getType())) {
           field.setAccessible(true);
           try {
             T constant = type.cast(field.get(null));
@@ -420,7 +439,6 @@ public final class ArbitraryInstances {
     }
 
     public static final class DeterministicRandom extends Random {
-      @SuppressWarnings("unused") // invoked by reflection
       public DeterministicRandom() {
         super(0);
       }
@@ -439,24 +457,28 @@ public final class ArbitraryInstances {
     }
 
     public static final class DummyRunnable implements Runnable, Serializable {
-      @Override public void run() {}
+      @Override
+      public void run() {}
     }
 
     public static final class DummyThreadFactory implements ThreadFactory, Serializable {
-      @Override public Thread newThread(Runnable r) {
+      @Override
+      public Thread newThread(Runnable r) {
         return new Thread(r);
       }
     }
 
     public static final class DummyExecutor implements Executor, Serializable {
-      @Override public void execute(Runnable command) {}
+      @Override
+      public void execute(Runnable command) {}
     }
   }
 
   private static final class NullByteSink extends ByteSink implements Serializable {
     private static final NullByteSink INSTANCE = new NullByteSink();
 
-    @Override public OutputStream openStream() {
+    @Override
+    public OutputStream openStream() {
       return ByteStreams.nullOutputStream();
     }
   }
@@ -464,14 +486,17 @@ public final class ArbitraryInstances {
   // Compare by toString() to satisfy 2 properties:
   // 1. compareTo(null) should throw NullPointerException
   // 2. the order is deterministic and easy to understand, for debugging purpose.
+  @SuppressWarnings("ComparableType")
   private static final class ByToString implements Comparable<Object>, Serializable {
     private static final ByToString INSTANCE = new ByToString();
 
-    @Override public int compareTo(Object o) {
+    @Override
+    public int compareTo(Object o) {
       return toString().compareTo(o.toString());
     }
 
-    @Override public String toString() {
+    @Override
+    public String toString() {
       return "BY_TO_STRING";
     }
 
@@ -484,11 +509,13 @@ public final class ArbitraryInstances {
   private static final class AlwaysEqual extends Ordering<Object> implements Serializable {
     private static final AlwaysEqual INSTANCE = new AlwaysEqual();
 
-    @Override public int compare(Object o1, Object o2) {
+    @Override
+    public int compare(Object o1, Object o2) {
       return 0;
     }
 
-    @Override public String toString() {
+    @Override
+    public String toString() {
       return "ALWAYS_EQUAL";
     }
 
