@@ -1,17 +1,15 @@
 /*
  * Copyright (C) 2009 The Guava Authors
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
+ * in compliance with the License. You may obtain a copy of the License at
  *
  * http://www.apache.org/licenses/LICENSE-2.0
  *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ * Unless required by applicable law or agreed to in writing, software distributed under the License
+ * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
+ * or implied. See the License for the specific language governing permissions and limitations under
+ * the License.
  */
 
 package com.google.common.reflect;
@@ -19,23 +17,25 @@ package com.google.common.reflect;
 import static com.google.common.base.Preconditions.checkArgument;
 import static com.google.common.base.Preconditions.checkNotNull;
 import static com.google.common.base.Preconditions.checkState;
+import static java.util.Arrays.asList;
 
 import com.google.common.annotations.Beta;
 import com.google.common.base.Joiner;
 import com.google.common.base.Objects;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.Maps;
-
 import java.lang.reflect.GenericArrayType;
 import java.lang.reflect.ParameterizedType;
 import java.lang.reflect.Type;
 import java.lang.reflect.TypeVariable;
 import java.lang.reflect.WildcardType;
 import java.util.Arrays;
+import java.util.LinkedHashSet;
 import java.util.Map;
+import java.util.Map.Entry;
+import java.util.Set;
 import java.util.concurrent.atomic.AtomicInteger;
-
-import javax.annotation.Nullable;
+import org.checkerframework.checker.nullness.qual.Nullable;
 
 /**
  * An object of this class encapsulates type mappings from type variables. Mappings are established
@@ -64,8 +64,31 @@ public final class TypeResolver {
     this.typeTable = typeTable;
   }
 
-  static TypeResolver accordingTo(Type type) {
-    return new TypeResolver().where(TypeMappingIntrospector.getTypeMappings(type));
+  /**
+   * Returns a resolver that resolves types "covariantly".
+   *
+   * <p>For example, when resolving {@code List<T>} in the context of {@code ArrayList<?>}, {@code
+   * <T>} is covariantly resolved to {@code <?>} such that return type of {@code List::get} is
+   * {@code <?>}.
+   */
+  static TypeResolver covariantly(Type contextType) {
+    return new TypeResolver().where(TypeMappingIntrospector.getTypeMappings(contextType));
+  }
+
+  /**
+   * Returns a resolver that resolves types "invariantly".
+   *
+   * <p>For example, when resolving {@code List<T>} in the context of {@code ArrayList<?>}, {@code
+   * <T>} cannot be invariantly resolved to {@code <?>} because otherwise the parameter type of
+   * {@code List::set} will be {@code <?>} and it'll falsely say any object can be passed into
+   * {@code ArrayList<?>::set}.
+   *
+   * <p>Instead, {@code <?>} will be resolved to a capture in the form of a type variable {@code
+   * <capture-of-? extends Object>}, effectively preventing {@code set} from accepting any type.
+   */
+  static TypeResolver invariantly(Type contextType) {
+    Type invariantContext = WildcardCapturer.INSTANCE.capture(contextType);
+    return new TypeResolver().where(TypeMappingIntrospector.getTypeMappings(invariantContext));
   }
 
   /**
@@ -81,11 +104,11 @@ public final class TypeResolver {
    * thereof.
    *
    * @param formal The type whose type variables or itself is mapped to other type(s). It's almost
-   *        always a bug if {@code formal} isn't a type variable and contains no type variable. Make
-   *        sure you are passing the two parameters in the right order.
+   *     always a bug if {@code formal} isn't a type variable and contains no type variable. Make
+   *     sure you are passing the two parameters in the right order.
    * @param actual The type that the formal type variable(s) are mapped to. It can be or contain yet
-   *        other type variables, in which case these type variables will be further resolved if
-   *        corresponding mappings exist in the current {@code TypeResolver} instance.
+   *     other type variables, in which case these type variables will be further resolved if
+   *     corresponding mappings exist in the current {@code TypeResolver} instance.
    */
   public TypeResolver where(Type formal, Type actual) {
     Map<TypeVariableKey, Type> mappings = Maps.newHashMap();
@@ -99,16 +122,22 @@ public final class TypeResolver {
   }
 
   private static void populateTypeMappings(
-      final Map<TypeVariableKey, Type> mappings, Type from, final Type to) {
+      final Map<TypeVariableKey, Type> mappings, final Type from, final Type to) {
     if (from.equals(to)) {
       return;
     }
     new TypeVisitor() {
-      @Override void visitTypeVariable(TypeVariable<?> typeVariable) {
+      @Override
+      void visitTypeVariable(TypeVariable<?> typeVariable) {
         mappings.put(new TypeVariableKey(typeVariable), to);
       }
-      @Override void visitWildcardType(WildcardType fromWildcardType) {
-        WildcardType toWildcardType = expectArgument(WildcardType.class, to);
+
+      @Override
+      void visitWildcardType(WildcardType fromWildcardType) {
+        if (!(to instanceof WildcardType)) {
+          return; // okay to say <?> is anything
+        }
+        WildcardType toWildcardType = (WildcardType) to;
         Type[] fromUpperBounds = fromWildcardType.getUpperBounds();
         Type[] toUpperBounds = toWildcardType.getUpperBounds();
         Type[] fromLowerBounds = fromWildcardType.getLowerBounds();
@@ -116,7 +145,9 @@ public final class TypeResolver {
         checkArgument(
             fromUpperBounds.length == toUpperBounds.length
                 && fromLowerBounds.length == toLowerBounds.length,
-            "Incompatible type: %s vs. %s", fromWildcardType, to);
+            "Incompatible type: %s vs. %s",
+            fromWildcardType,
+            to);
         for (int i = 0; i < fromUpperBounds.length; i++) {
           populateTypeMappings(mappings, fromUpperBounds[i], toUpperBounds[i]);
         }
@@ -124,35 +155,61 @@ public final class TypeResolver {
           populateTypeMappings(mappings, fromLowerBounds[i], toLowerBounds[i]);
         }
       }
-      @Override void visitParameterizedType(ParameterizedType fromParameterizedType) {
+
+      @Override
+      void visitParameterizedType(ParameterizedType fromParameterizedType) {
+        if (to instanceof WildcardType) {
+          return; // Okay to say Foo<A> is <?>
+        }
         ParameterizedType toParameterizedType = expectArgument(ParameterizedType.class, to);
-        checkArgument(fromParameterizedType.getRawType().equals(toParameterizedType.getRawType()),
-            "Inconsistent raw type: %s vs. %s", fromParameterizedType, to);
+        if (fromParameterizedType.getOwnerType() != null
+            && toParameterizedType.getOwnerType() != null) {
+          populateTypeMappings(
+              mappings, fromParameterizedType.getOwnerType(), toParameterizedType.getOwnerType());
+        }
+        checkArgument(
+            fromParameterizedType.getRawType().equals(toParameterizedType.getRawType()),
+            "Inconsistent raw type: %s vs. %s",
+            fromParameterizedType,
+            to);
         Type[] fromArgs = fromParameterizedType.getActualTypeArguments();
         Type[] toArgs = toParameterizedType.getActualTypeArguments();
-        checkArgument(fromArgs.length == toArgs.length,
-            "%s not compatible with %s", fromParameterizedType, toParameterizedType);
+        checkArgument(
+            fromArgs.length == toArgs.length,
+            "%s not compatible with %s",
+            fromParameterizedType,
+            toParameterizedType);
         for (int i = 0; i < fromArgs.length; i++) {
           populateTypeMappings(mappings, fromArgs[i], toArgs[i]);
         }
       }
-      @Override void visitGenericArrayType(GenericArrayType fromArrayType) {
+
+      @Override
+      void visitGenericArrayType(GenericArrayType fromArrayType) {
+        if (to instanceof WildcardType) {
+          return; // Okay to say A[] is <?>
+        }
         Type componentType = Types.getComponentType(to);
         checkArgument(componentType != null, "%s is not an array type.", to);
         populateTypeMappings(mappings, fromArrayType.getGenericComponentType(), componentType);
       }
-      @Override void visitClass(Class<?> fromClass) {
-        // Can't map from a raw class to anything other than itself.
+
+      @Override
+      void visitClass(Class<?> fromClass) {
+        if (to instanceof WildcardType) {
+          return; // Okay to say Foo is <?>
+        }
+        // Can't map from a raw class to anything other than itself or a wildcard.
         // You can't say "assuming String is Integer".
-        // And we don't support "assuming String is T"; user has to say "assuming T is String". 
-        throw new IllegalArgumentException("No type mapping from " + fromClass);
+        // And we don't support "assuming String is T"; user has to say "assuming T is String".
+        throw new IllegalArgumentException("No type mapping from " + fromClass + " to " + to);
       }
     }.visit(from);
   }
 
   /**
-   * Resolves all type variables in {@code type} and all downstream types and
-   * returns a corresponding type with type variables resolved.
+   * Resolves all type variables in {@code type} and all downstream types and returns a
+   * corresponding type with type variables resolved.
    */
   public Type resolveType(Type type) {
     checkNotNull(type);
@@ -170,6 +227,13 @@ public final class TypeResolver {
     }
   }
 
+  Type[] resolveTypesInPlace(Type[] types) {
+    for (int i = 0; i < types.length; i++) {
+      types[i] = resolveType(types[i]);
+    }
+    return types;
+  }
+
   private Type[] resolveTypes(Type[] types) {
     Type[] result = new Type[types.length];
     for (int i = 0; i < types.length; i++) {
@@ -181,8 +245,7 @@ public final class TypeResolver {
   private WildcardType resolveWildcardType(WildcardType type) {
     Type[] lowerBounds = type.getLowerBounds();
     Type[] upperBounds = type.getUpperBounds();
-    return new Types.WildcardTypeImpl(
-        resolveTypes(lowerBounds), resolveTypes(upperBounds));
+    return new Types.WildcardTypeImpl(resolveTypes(lowerBounds), resolveTypes(upperBounds));
   }
 
   private Type resolveGenericArrayType(GenericArrayType type) {
@@ -213,11 +276,11 @@ public final class TypeResolver {
   /** A TypeTable maintains mapping from {@link TypeVariable} to types. */
   private static class TypeTable {
     private final ImmutableMap<TypeVariableKey, Type> map;
-  
+
     TypeTable() {
       this.map = ImmutableMap.of();
     }
-    
+
     private TypeTable(ImmutableMap<TypeVariableKey, Type> map) {
       this.map = map;
     }
@@ -226,7 +289,7 @@ public final class TypeResolver {
     final TypeTable where(Map<TypeVariableKey, ? extends Type> mappings) {
       ImmutableMap.Builder<TypeVariableKey, Type> builder = ImmutableMap.builder();
       builder.putAll(map);
-      for (Map.Entry<TypeVariableKey, ? extends Type> mapping : mappings.entrySet()) {
+      for (Entry<TypeVariableKey, ? extends Type> mapping : mappings.entrySet()) {
         TypeVariableKey variable = mapping.getKey();
         Type type = mapping.getValue();
         checkArgument(!variable.equalsType(type), "Type variable %s bound to itself", variable);
@@ -237,15 +300,16 @@ public final class TypeResolver {
 
     final Type resolve(final TypeVariable<?> var) {
       final TypeTable unguarded = this;
-      TypeTable guarded = new TypeTable() {
-        @Override public Type resolveInternal(
-            TypeVariable<?> intermediateVar, TypeTable forDependent) {
-          if (intermediateVar.getGenericDeclaration().equals(var.getGenericDeclaration())) {
-            return intermediateVar;
-          }
-          return unguarded.resolveInternal(intermediateVar, forDependent);
-        }
-      };
+      TypeTable guarded =
+          new TypeTable() {
+            @Override
+            public Type resolveInternal(TypeVariable<?> intermediateVar, TypeTable forDependent) {
+              if (intermediateVar.getGenericDeclaration().equals(var.getGenericDeclaration())) {
+                return intermediateVar;
+              }
+              return unguarded.resolveInternal(intermediateVar, forDependent);
+            }
+          };
       return resolveInternal(var, guarded);
     }
 
@@ -307,27 +371,27 @@ public final class TypeResolver {
 
   private static final class TypeMappingIntrospector extends TypeVisitor {
 
-    private static final WildcardCapturer wildcardCapturer = new WildcardCapturer();
-
     private final Map<TypeVariableKey, Type> mappings = Maps.newHashMap();
 
     /**
-     * Returns type mappings using type parameters and type arguments found in
-     * the generic superclass and the super interfaces of {@code contextClass}.
+     * Returns type mappings using type parameters and type arguments found in the generic
+     * superclass and the super interfaces of {@code contextClass}.
      */
-    static ImmutableMap<TypeVariableKey, Type> getTypeMappings(
-        Type contextType) {
+    static ImmutableMap<TypeVariableKey, Type> getTypeMappings(Type contextType) {
+      checkNotNull(contextType);
       TypeMappingIntrospector introspector = new TypeMappingIntrospector();
-      introspector.visit(wildcardCapturer.capture(contextType));
+      introspector.visit(contextType);
       return ImmutableMap.copyOf(introspector.mappings);
     }
 
-    @Override void visitClass(Class<?> clazz) {
+    @Override
+    void visitClass(Class<?> clazz) {
       visit(clazz.getGenericSuperclass());
       visit(clazz.getGenericInterfaces());
     }
 
-    @Override void visitParameterizedType(ParameterizedType parameterizedType) {
+    @Override
+    void visitParameterizedType(ParameterizedType parameterizedType) {
       Class<?> rawClass = (Class<?>) parameterizedType.getRawType();
       TypeVariable<?>[] vars = rawClass.getTypeParameters();
       Type[] typeArgs = parameterizedType.getActualTypeArguments();
@@ -339,11 +403,13 @@ public final class TypeResolver {
       visit(parameterizedType.getOwnerType());
     }
 
-    @Override void visitTypeVariable(TypeVariable<?> t) {
+    @Override
+    void visitTypeVariable(TypeVariable<?> t) {
       visit(t.getBounds());
     }
 
-    @Override void visitWildcardType(WildcardType t) {
+    @Override
+    void visitWildcardType(WildcardType t) {
       visit(t.getUpperBounds());
     }
 
@@ -378,11 +444,21 @@ public final class TypeResolver {
   // }
   // Holder<List<?>> should *not* resolve the set() method to set(List<?> data).
   // Instead, it should create a capture of the wildcard so that set() rejects any List<T>.
-  private static final class WildcardCapturer {
+  private static class WildcardCapturer {
 
-    private final AtomicInteger id = new AtomicInteger();
+    static final WildcardCapturer INSTANCE = new WildcardCapturer();
 
-    Type capture(Type type) {
+    private final AtomicInteger id;
+
+    private WildcardCapturer() {
+      this(new AtomicInteger());
+    }
+
+    private WildcardCapturer(AtomicInteger id) {
+      this.id = id;
+    }
+
+    final Type capture(Type type) {
       checkNotNull(type);
       if (type instanceof Class) {
         return type;
@@ -392,24 +468,27 @@ public final class TypeResolver {
       }
       if (type instanceof GenericArrayType) {
         GenericArrayType arrayType = (GenericArrayType) type;
-        return Types.newArrayType(capture(arrayType.getGenericComponentType()));
+        return Types.newArrayType(
+            notForTypeVariable().capture(arrayType.getGenericComponentType()));
       }
       if (type instanceof ParameterizedType) {
         ParameterizedType parameterizedType = (ParameterizedType) type;
+        Class<?> rawType = (Class<?>) parameterizedType.getRawType();
+        TypeVariable<?>[] typeVars = rawType.getTypeParameters();
+        Type[] typeArgs = parameterizedType.getActualTypeArguments();
+        for (int i = 0; i < typeArgs.length; i++) {
+          typeArgs[i] = forTypeVariable(typeVars[i]).capture(typeArgs[i]);
+        }
         return Types.newParameterizedTypeWithOwner(
-            captureNullable(parameterizedType.getOwnerType()),
-            (Class<?>) parameterizedType.getRawType(),
-            capture(parameterizedType.getActualTypeArguments()));
+            notForTypeVariable().captureNullable(parameterizedType.getOwnerType()),
+            rawType,
+            typeArgs);
       }
       if (type instanceof WildcardType) {
         WildcardType wildcardType = (WildcardType) type;
         Type[] lowerBounds = wildcardType.getLowerBounds();
         if (lowerBounds.length == 0) { // ? extends something changes to capture-of
-          Type[] upperBounds = wildcardType.getUpperBounds();
-          String name = "capture#" + id.incrementAndGet() + "-of ? extends "
-              + Joiner.on('&').join(upperBounds);
-          return Types.newArtificialTypeVariable(
-              WildcardCapturer.class, name, wildcardType.getUpperBounds());
+          return captureAsTypeVariable(wildcardType.getUpperBounds());
         } else {
           // TODO(benyu): handle ? super T somehow.
           return type;
@@ -418,20 +497,42 @@ public final class TypeResolver {
       throw new AssertionError("must have been one of the known types");
     }
 
+    TypeVariable<?> captureAsTypeVariable(Type[] upperBounds) {
+      String name =
+          "capture#" + id.incrementAndGet() + "-of ? extends " + Joiner.on('&').join(upperBounds);
+      return Types.newArtificialTypeVariable(WildcardCapturer.class, name, upperBounds);
+    }
+
+    private WildcardCapturer forTypeVariable(final TypeVariable<?> typeParam) {
+      return new WildcardCapturer(id) {
+        @Override
+        TypeVariable<?> captureAsTypeVariable(Type[] upperBounds) {
+          Set<Type> combined = new LinkedHashSet<>(asList(upperBounds));
+          // Since this is an artifically generated type variable, we don't bother checking
+          // subtyping between declared type bound and actual type bound. So it's possible that we
+          // may generate something like <capture#1-of ? extends Foo&SubFoo>.
+          // Checking subtype between declared and actual type bounds
+          // adds recursive isSubtypeOf() call and feels complicated.
+          // There is no contract one way or another as long as isSubtypeOf() works as expected.
+          combined.addAll(asList(typeParam.getBounds()));
+          if (combined.size() > 1) { // Object is implicit and only useful if it's the only bound.
+            combined.remove(Object.class);
+          }
+          return super.captureAsTypeVariable(combined.toArray(new Type[0]));
+        }
+      };
+    }
+
+    private WildcardCapturer notForTypeVariable() {
+      return new WildcardCapturer(id);
+    }
+
     private Type captureNullable(@Nullable Type type) {
       if (type == null) {
         return null;
       }
       return capture(type);
     }
-
-    private Type[] capture(Type[] types) {
-      Type[] result = new Type[types.length];
-      for (int i = 0; i < types.length; i++) {
-        result[i] = capture(types[i]);
-      }
-      return result;
-    }
   }
 
   /**
@@ -439,14 +540,13 @@ public final class TypeResolver {
    * they are declared by the same {@link java.lang.reflect.GenericDeclaration} and have the same
    * name, even if their bounds differ.
    *
-   * <p>While resolving a type variable from a {var -> type} map, we don't care whether the
+   * <p>While resolving a type variable from a {@code var -> type} map, we don't care whether the
    * type variable's bound has been partially resolved. As long as the type variable "identity"
    * matches.
    *
-   * <p>On the other hand, if for example we are resolving List<A extends B> to
-   * List<A extends String>, we need to compare that <A extends B> is unequal to
-   * <A extends String> in order to decide to use the transformed type instead of the original
-   * type.
+   * <p>On the other hand, if for example we are resolving {@code List<A extends B>} to {@code
+   * List<A extends String>}, we need to compare that {@code <A extends B>} is unequal to {@code <A
+   * extends String>} in order to decide to use the transformed type instead of the original type.
    */
   static final class TypeVariableKey {
     private final TypeVariable<?> var;
@@ -455,11 +555,13 @@ public final class TypeResolver {
       this.var = checkNotNull(var);
     }
 
-    @Override public int hashCode() {
+    @Override
+    public int hashCode() {
       return Objects.hashCode(var.getGenericDeclaration(), var.getName());
     }
 
-    @Override public boolean equals(Object obj) {
+    @Override
+    public boolean equals(Object obj) {
       if (obj instanceof TypeVariableKey) {
         TypeVariableKey that = (TypeVariableKey) obj;
         return equalsTypeVariable(that.var);
@@ -468,12 +570,13 @@ public final class TypeResolver {
       }
     }
 
-    @Override public String toString() {
+    @Override
+    public String toString() {
       return var.toString();
     }
 
     /** Wraps {@code t} in a {@code TypeVariableKey} if it's a type variable. */
-    static Object forLookup(Type t) {
+    static TypeVariableKey forLookup(Type t) {
       if (t instanceof TypeVariable) {
         return new TypeVariableKey((TypeVariable<?>) t);
       } else {
@@ -482,8 +585,8 @@ public final class TypeResolver {
     }
 
     /**
-     * Returns true if {@code type} is a {@code TypeVariable} with the same name and declared by
-     * the same {@code GenericDeclaration}.
+     * Returns true if {@code type} is a {@code TypeVariable} with the same name and declared by the
+     * same {@code GenericDeclaration}.
      */
     boolean equalsType(Type type) {
       if (type instanceof TypeVariable) {
